@@ -5,10 +5,45 @@ use strict;
 
 use XML::Atom;
 use vars qw( @EXPORT_OK @ISA );
+use Encode;
 use Exporter;
-@EXPORT_OK = qw( first nodelist textValue iso2dt encode_xml remove_default_ns );
+@EXPORT_OK = qw( set_ns hack_unicode_entity first nodelist textValue iso2dt encode_xml remove_default_ns );
 @ISA = qw( Exporter );
 
+our %NS_MAP = (
+    '0.3' => 'http://purl.org/atom/ns#',
+    '1.0' => 'http://www.w3.org/2005/Atom',
+);
+
+our %NS_VERSION = reverse %NS_MAP;
+
+sub set_ns {
+    my $thing = shift;
+    my($param) = @_;
+    if ($param->{Namespace}) {
+        $thing->{ns} = $param->{Namespace};
+        $thing->{version} = $NS_VERSION{$param->{Namespace}};
+    } else  {
+        my $version = $param->{Version} || '0.3';
+        my $ns = $NS_MAP{$version} or $thing->error("Unknown version: $version");
+        $thing->{ns} = $ns;
+        $thing->{version} = $version;
+    }
+}
+
+sub ns_to_version {
+    my $ns = shift;
+    $NS_VERSION{$ns};
+}
+
+sub hack_unicode_entity {
+    my $data = shift;
+    Encode::_utf8_on($data);
+    $data =~ s/&#x(\w{4});/chr(hex($1))/eg;
+    Encode::_utf8_off($data);
+    $data;
+}
+
 sub first {
     my @nodes = nodelist(@_);
     return unless @nodes;
